#!/usr/bin/perl
# This script is for checking, adding and removing an IP Alias name
# as defined by $ALIAS_FILE.
#
# The aliases are defined with the following syntax
# Nic | Alias IP | NetMask | Partner | Primary
#
# Clark.Cooper@vc3.com
# 05/11/00

# The Net perl module Ping.pm is required!!! 
# You may download via http://www.cpan.org.
use Net::Ping;

$ALIAS_FILE="/etc/hostname.alias";
$PING_COUNT="10";
$UNAME="/bin/uname";
$OS=`$UNAME -s`;
chop $OS;
$IFCONFIG="/sbin/ifconfig";
$LOGGER="/usr/bin/logger";	# System Logger (/var/adm/messages)
$CHECK_INTERVAL="20";		# When in Monitor mode, how often in seconds we
                                # check to see if the Alias IP is alive.


&GetConfiguration;

# Act on passed parameters. 
SWITCH:{

	if ( (@ARGV[0] eq "AddAlias") || (@ARGV[0] eq "RemoveAlias") ){
		&ForEachInterface;
		last SWITCH;
	}

	if (@ARGV[0] eq "Monitor") {

		# Check to see if we have any existing processes already running.
		if ( &ExistingPID eq "true" ){
			print "Alias Defender is already running in Monitor mode..\n";
			exit 1;
		}

		# Monitor the aliases indefinitely.
		while (){
			&ForEachInterface;
			sleep $CHECK_INTERVAL;
		}
		last SWITCH;
	}

	print "Usage: $0 { AddAlias [Alias IP] | Monitor | RemoveAlias [Alias IP] }\n";
	exit 1;
}

exit 0;

###################################################################
####################### Subroutines ###############################
###################################################################

sub GetConfiguration{
	# Subroutine for reading the configuration file for determining what
        # Alias IP's map to which interfaces.  Also makes subroutine calls for
        # adding and removing Alias.  Creates the array $INTERFACE_CARD.

	open(ALIAS_FILE,"<$ALIAS_FILE") || die "ERROR! Could not open $ALIAS_FILE for reading.\n";
	while ( $wholeline=<ALIAS_FILE> ){

		#skip if comment or blank line
		next if ( ($wholeline =~ /^#/) || ($wholeline =~ /^\s/) );

		$wholeline =~ tr/A-Z/a-z/;		# upper case to lower case
		$wholeline =~ s/\s+//g;			# remove white space
	
		$INTERFACE_CARD{"$wholeline"}="";
	}
	close ALIAS_FILE;

	# Check to make sure we retrieved a defined interface card.
	$SIZE_OF_INTERFACE_CARD = %INTERFACE_CARD;
	if ( $SIZE_OF_INTERFACE_CARD == 0 ){
		&DisplayLogMessage("ERROR: No interfaces were found configured in /etc/hostname.alias.");
		exit 1;
	}

	# Get the default router.
	SWITCH_GetConfiguration:{
		if ( $OS eq "SunOS"){
			open(DEFAULT_ROUTER,"</etc/defaultrouter") || die "ERROR!! Could not open /etc/defaultrouter for reading.\n";
			@DEFAULT_ROUTER=<DEFAULT_ROUTER>;
			close DEFAULT_ROUTER;

			$ROUTER=@DEFAULT_ROUTER[0];

			last SWITCH_GetConfiguration;
		}

		if ( $OS eq "Linux"){
			open(DEFAULT_ROUTER,"</etc/sysconfig/network") || die "ERROR!!  Could not open /etc/sysconfig/network for reading.\n";
			@DEFAULT_ROUTER=<DEFAULT_ROUTER>;
			close DEFAULT_ROUTER;

			@ROUTER_ARRAY=grep(/^GATEWAY/,@DEFAULT_ROUTER);
			($JUNK,$ROUTER) = split(/\=/,@ROUTER_ARRAY[0]);

			last SWITCH_GetConfiguration;
		}

		print "ERROR!! Do not support the Operating System $OS.\n";
		exit 1;
	}
	chop $ROUTER;
}


sub ForEachInterface{
	# Rountine for stepping through each defined Interface in /etc/hostname.alias.

	foreach $nic (keys %INTERFACE_CARD){
		( $INTERFACE,$ALIAS_IP,$ALIAS_NETMASK,$PARTNER,$PRIMARY ) = split (/\|/,$nic);

		# Check to make sure alias configuration is good.
		&CheckConfiguration;
	
		# Skip if a second parameter is passed and is not $ALIAS_IP.
		next if ( (@ARGV[1] ne "") && (@ARGV[1] ne "$ALIAS_IP") );

		# Skip if we are to add an alias and the alias is alive as 
		# determined by subroutine CheckAlias who returns false if alive.
		next if ( ((@ARGV[0] eq "AddAlias") || (@ARGV[0] eq "Monitor")) && (&CheckAlias eq "false") );

		if ( (@ARGV[0] eq "AddAlias") || (@ARGV[0] eq "Monitor") ){
			&AddAlias;
		}else{
			&RemoveAlias;
		}
	}
}
			

	
sub CheckConfiguration{
	# Check the alias configuration as retrieved from $ALIAS_FILE.

	if ( $ROUTER eq "" ){
		&DisplayLogMessage("ERROR: Was unable to retrieve the default router.");
		exit 1;
	}

	if ( ($INTERFACE eq "") || ($ALIAS_IP eq "") || ($ALIAS_NETMASK eq "") || 
	($PARTNER eq "") || ($PRIMARY eq "") ){
		&DisplayLogMessage("ERROR!! Could not determine alias information.   Check /etc/hostname.alias for correct syntax.");
		exit 1;
	}
	
}

sub DisplayLogMessage{
	# Subroutine for displaying contents of $MESSAGE to STDOUT
        # and also for logging $MESSAGE into /var/adm/messages.

	local ($MESSAGE) = @_;

	print "$MESSAGE\n";
	`$LOGGER -p daemon.notice $MESSAGE`;
}

sub CheckAlias{
	# Subroutine for checking to see if the alias is alive.
	# A return value of "true" means that we should add the alias.
	#
	# For a return value of true, the following must be meet:
        #       if we are the primary
        #               the alias must be down and router must be up.
        #
        #       if we are not the primary
        #               the alias and partner must be down while the router must be up.
        #


	if (&PingHost("$ALIAS_IP") eq "true"){
		return false;
	}else{
		if (&PingHost("$ROUTER") eq "true"){
			if ($PRIMARY eq "true"){
				# Alias is down, router is up and we are the primary.
                                # Do not care if partner is alive. Bring up alias.
                                return true;
			}else{
				if (&PingHost("$PARTNER") eq "true"){
					&DisplayLogMessage("WARNING!! Could not failover.  $ALIAS_IP is NOT alive but the primary $PARTNER is alive.");
					return false;
				}else{
					# Alias is down, router is up, partner is down
                                        # and we are are the secondary.  Bring up alias.
                                        return true;
				}
			}
		}else{
			# Router is down.  In this situation, a failover is not done because of
			# the possibility the switch or network is down.
			&DisplayLogMessage("ERROR!! Could not bring up $ALIAS_IP due to router unavailability.   Switch/Network down?");

			return false;
		}
	}

}

sub PingHost{
	# Subroutine for checking the availablity of a passed host.
	# Returns true if host is alive or false if dead.
	#
	# NOTE: Specifying the ``icmp'' protocol requires that the program 
	#       be run as root or that the program be setuid to root. 
	
	local($host)=@_;

	$p = Net::Ping->new("icmp");
	if ($p->ping($host,$PING_COUNT)){
       		return true;
	}else{
       		return false;
	}
	undef($p);
}

					
sub AddAlias{
	open(IFCONFIG,"$IFCONFIG -a|");
	@IFCONFIG=<IFCONFIG>;
	close IFCONFIG;

	# Check to make sure if the alias is already binded.
	# Return if binded.
	return if ( grep(/$ALIAS_IP/,@IFCONFIG) );

	# Solaris systems must use the plumb command to create the interface.
	if ( $OS eq "SunOS" ){
		`$IFCONFIG $INTERFACE plumb`;
	}

	`$IFCONFIG $INTERFACE $ALIAS_IP netmask $ALIAS_NETMASK`;
	`$IFCONFIG $INTERFACE up`;

	if ( $? == 0 ){
		&DisplayLogMessage("Alias $ALIAS_IP was successfully added to interface $INTERFACE.");
	}else{
		&DisplayLogMessage("ERROR!! Was unable to add $ALIAS_IP to interface $INTERFACE.");
	}
}

sub RemoveAlias{
	# Routine for termininating an alias IP.

	open(IFCONFIG,"$IFCONFIG -a|");
	@IFCONFIG=<IFCONFIG>;
	close IFCONFIG;

	# Check to make sure that the alias is binded.
	# Return if not binded.
	return if ( !grep(/$ALIAS_IP/,@IFCONFIG) );

	# Determine the correct syntax for brining down an interface.
        SWITCH_RemoveAlias:{
                if ( $OS eq "SunOS" ){

			# Solaris 2.6 and Solaris 7 vs Solaris 8 have different 
			# ways of removing an interface.
			$SUN_VERSION=`$UNAME -r`;
			$SUN_VERSION =~ s/\s+//g;
			SWITCH_SUN_VERSION:{
				if ( ($SUN_VERSION == 5.6) || ($SUN_VERSION == 5.7) ){
                        		$DOWN_INTERFACE="$IFCONFIG $INTERFACE 0 down";
					last SWITCH_SUN_VERSION;
				}
				if ( $SUN_VERSION >= 5.8 ){
                        		$DOWN_INTERFACE="$IFCONFIG $INTERFACE unplumb";
					last SWITCH_SUN_VERSION;
				}
			}
                        last SWITCH_RemoveAlias;
                }

                if ( $OS eq "Linux" ){
			$DOWN_INTERFACE="$IFCONFIG $INTERFACE $ALIAS_IP down";
                        last SWITCH_RemoveAlias;
                }
        }

	$RETURN=`$DOWN_INTERFACE`;
	if ( $? == 0 ){
		&DisplayLogMessage("Alias $ALIAS_IP has been removed from $INTERFACE.");

	}else{
		&DisplayLogMessage("ERROR!! Was unable to remove alias $ALIAS_IP from $INTERFACE.\n");
	}
}

sub ExistingPID{
	# Check to see if there are any outstanding "Alias Defender" processes running.
	# Return true if there are existing processes.

	local ($PIDS);

	$PID="none";

	# Determine the best syntax for a process listing.
	SWITCH_ExistingPID:{
		if ( $OS eq "SunOS" ){
			$pscmd="ps -ef";
			$PID_FIELD="2";
			last SWITCH_ExistingPID;
		}

		if ( $OS eq "Linux" ){
			$pscmd="ps -ax";
			$PID_FIELD="1";
			last SWITCH_ExistingPID;
		}
	}

	open(PS,"$pscmd|") || warn "WARNING!! Could not run $pscmd: $!\n";
	@PS=<PS>;
	close PS;

	# Put into an array, the full process listing of any processes
	# which are running as "$0 Monitor".
	@RUNNING_PIDS=grep(/$0 Monitor/,@PS);

	# Step through each process in @RUNNING_PIDS.
	foreach $pid (@RUNNING_PIDS){
		( @ExistingPIDs ) = split(/\s+/,$pid );

		# If the process id is not our current process.
		if ( (@ExistingPIDs[$PID_FIELD] != $$ ) && (@ExistingPIDs[$PID_FIELD] ne "") ){
			return true;
		}
	}
	
	# No previously existing monitor processes were detected.
	return false;
}
